@@ -4,6 +4,7 @@ mod system_properties;
 use crate::extra;
 use crate::shared;
 use crate::traits::*;
+use byte_unit::AdjustedByte;
 use itertools::Itertools;
 use std::ffi::{CStr, CString};
 use std::fs;
@@ -77,6 +78,10 @@ impl BatteryReadout for AndroidBatteryReadout {
             ))),
         }
     }
+
+    fn health(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 impl KernelReadout for AndroidKernelReadout {
@@ -123,18 +128,29 @@ impl GeneralReadout for AndroidGeneralReadout {
         }
     }
 
+    fn backlight(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn resolution(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn machine(&self) -> Result<String, ReadoutError> {
         let product_readout = AndroidProductReadout::new();
 
-        let vendor = product_readout.vendor()?;
         let family = product_readout.family()?;
+        let vendor = product_readout.vendor()?;
         let product = product_readout.product()?;
 
-        let product = format!("{} {} ({})", vendor, family, product);
-        let new_product: Vec<_> = product.split_whitespace().into_iter().unique().collect();
+        let new_product = format!("{} {} {}", vendor, family, product);
 
         if product.is_empty() || product.len() <= 15 {
-            return Ok(new_product.into_iter().join(" "));
+            return Ok(new_product
+                .split_whitespace()
+                .into_iter()
+                .unique()
+                .join(" "));
         }
 
         Ok(product)
@@ -155,6 +171,26 @@ impl GeneralReadout for AndroidGeneralReadout {
         }
     }
 
+    fn distribution(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn desktop_environment(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn session(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn window_manager(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn terminal(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn shell(&self, format: ShellFormat, kind: ShellKind) -> Result<String, ReadoutError> {
         if let Some(shell) = std::env::var_os("SHELL") {
             if let Some(relative) = PathBuf::from(shell).file_name() {
@@ -164,7 +200,7 @@ impl GeneralReadout for AndroidGeneralReadout {
             }
         }
 
-        return shared::shell(format, kind);
+        shared::shell(format, kind)
     }
 
     fn cpu_model_name(&self) -> Result<String, ReadoutError> {
@@ -251,6 +287,14 @@ impl GeneralReadout for AndroidGeneralReadout {
             ))
         }
     }
+
+    fn os_name(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn disk_space(&self) -> Result<(AdjustedByte, AdjustedByte), ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 impl MemoryReadout for AndroidMemoryReadout {
@@ -324,43 +368,18 @@ impl ProductReadout for AndroidProductReadout {
     }
 
     fn family(&self) -> Result<String, ReadoutError> {
-        getprop("ro.product.model").ok_or(ReadoutError::Other("getprop failed".to_string()))
-        // ro.product.model
-        // ro.product.odm.model
-        // ro.product.product.model
-        // ro.product.system.model
-        // ro.product.system_ext.model
-        // ro.product.vendor.model
-        // Same in all cases ( needs more testing in other devices )
+        getprop("ro.product.model")
+            .ok_or_else(|| ReadoutError::Other("Failed to get device family property".to_string()))
     }
 
     fn vendor(&self) -> Result<String, ReadoutError> {
-        getprop("ro.product.brand").ok_or(ReadoutError::Other("getprop failed".to_string()))
-        // ro.product.brand
-        // ro.product.manufacturer
-        // ro.product.odm.brand
-        // ro.product.odm.manufacturer
-        // ro.product.product.brand
-        // ro.product.product.manufacturer
-        // ro.product.system.brand
-        // ro.product.system.manufacturer
-        // ro.product.system_ext.brand
-        // ro.product.system_ext.manufacturer
-        // ro.product.vendor.brand
-        // ro.product.vendor.manufacturer
-        // Same in all cases ( needs more testing in other devices )
+        getprop("ro.product.brand")
+            .ok_or_else(|| ReadoutError::Other("Failed to get device vendor property".to_string()))
     }
 
     fn product(&self) -> Result<String, ReadoutError> {
-        getprop("ro.build.product").ok_or(ReadoutError::Other("getprop failed".to_string()))
-        // ro.build.product
-        // ro.product.device
-        // ro.product.odm.device
-        // ro.product.product.device
-        // ro.product.system.device
-        // ro.product.system_ext.device
-        // ro.product.vendor.device
-        // Same in all cases ( needs more testing in other devices )
+        getprop("ro.build.product")
+            .ok_or_else(|| ReadoutError::Other("Failed to get device product property".to_string()))
     }
 }
 
@@ -418,25 +437,14 @@ impl AndroidPackageReadout {
         };
 
         let dpkg_dir = Path::new(&prefix).join("var/lib/dpkg/info");
-        let dir_entries = extra::list_dir_entries(&dpkg_dir);
-
-        if !dir_entries.is_empty() {
-            return Some(
-                dir_entries
-                    .iter()
-                    .filter(|x| {
-                        if let Some(ext) = extra::path_extension(x) {
-                            ext == "list"
-                        } else {
-                            false
-                        }
-                    })
-                    .into_iter()
-                    .count(),
-            );
-        }
-
-        None
+        
+        extra::get_entries(&dpkg_dir).map(|entries| {
+            entries
+                .iter()
+                .filter(|x| extra::path_extension(x).unwrap_or_default() == "list")
+                .into_iter()
+                .count()
+        })
     }
 
     /// Returns the number of installed packages for systems
@@ -451,7 +459,27 @@ impl NetworkReadout for AndroidNetworkReadout {
         AndroidNetworkReadout
     }
 
+    fn tx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn tx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn logical_address(&self, interface: Option<&str>) -> Result<String, ReadoutError> {
         shared::logical_address(interface)
     }
+
+    fn physical_address(&self, _: Option<&str>) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
