@@ -4,7 +4,7 @@ use crate::traits::*;
 use crate::winman;
 use byte_unit::AdjustedByte;
 use std::fs;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use sysctl::{Ctl, Sysctl};
 
 impl From<sqlite::Error> for ReadoutError {
@@ -68,9 +68,9 @@ impl BatteryReadout for FreeBSDBatteryReadout {
                         1 => return Ok(BatteryState::Discharging),
                         2 => return Ok(BatteryState::Charging),
                         _ => {
-                            return Err(ReadoutError::Other(format!(
-                                "An unsupported battery state was reported."
-                            )))
+                            return Err(ReadoutError::Other(
+                                "An unsupported battery state was reported.".to_string(),
+                            ))
                         }
                     };
                 }
@@ -79,6 +79,10 @@ impl BatteryReadout for FreeBSDBatteryReadout {
 
         Err(ReadoutError::MetricNotAvailable)
     }
+
+    fn health(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 impl KernelReadout for FreeBSDKernelReadout {
@@ -205,7 +209,7 @@ impl GeneralReadout for FreeBSDGeneralReadout {
                         .join("status");
 
                     if let Ok(status) = fs::read_to_string(path) {
-                        if let Some(name) = status.split_whitespace().nth(0) {
+                        if let Some(name) = status.split_whitespace().next() {
                             terminal_name = name.to_string();
                         }
                     }
@@ -220,7 +224,9 @@ impl GeneralReadout for FreeBSDGeneralReadout {
         let terminal = terminal_name();
 
         if terminal.is_empty() {
-            return Err(ReadoutError::Other(format!("Could not to fetch terminal.")));
+            return Err(ReadoutError::Other(
+                "Could not to fetch terminal.".to_string(),
+            ));
         }
 
         Ok(terminal)
@@ -305,6 +311,18 @@ impl MemoryReadout for FreeBSDMemoryReadout {
             / 1024)
     }
 
+    fn buffers(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn cached(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn reclaimable(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn used(&self) -> Result<u64, ReadoutError> {
         let total = self.total().unwrap();
         let free = self.free().unwrap();
@@ -339,16 +357,12 @@ impl PackageReadout for FreeBSDPackageReadout {
     fn count_pkgs(&self) -> Vec<(PackageManager, usize)> {
         let mut packages = Vec::new();
 
-        if extra::which("pkg") {
-            if let Some(c) = FreeBSDPackageReadout::count_pkg() {
-                packages.push((PackageManager::Pkg, c));
-            }
+        if let Some(c) = FreeBSDPackageReadout::count_pkg() {
+            packages.push((PackageManager::Pkg, c));
         }
 
-        if extra::which("cargo") {
-            if let Some(c) = FreeBSDPackageReadout::count_cargo() {
-                packages.push((PackageManager::Cargo, c));
-            }
+        if let Some(c) = FreeBSDPackageReadout::count_cargo() {
+            packages.push((PackageManager::Cargo, c));
         }
 
         packages
@@ -357,10 +371,14 @@ impl PackageReadout for FreeBSDPackageReadout {
 
 impl FreeBSDPackageReadout {
     fn count_pkg() -> Option<usize> {
-        let connection = sqlite::open("/var/db/pkg/local.sqlite");
+        let db = "/var/db/pkg/local.sqlite";
+        if !Path::new(db).exists() {
+            return None;
+        }
 
+        let connection = sqlite::open(db);
         if let Ok(con) = connection {
-            let statement = con.prepare("select count(*) from packages");
+            let statement = con.prepare("SELECT COUNT(*) FROM packages");
             if let Ok(mut s) = statement {
                 if s.next().is_ok() {
                     return match s.read::<Option<i64>>(0) {
@@ -384,7 +402,27 @@ impl NetworkReadout for FreeBSDNetworkReadout {
         FreeBSDNetworkReadout
     }
 
+    fn tx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn tx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn logical_address(&self, interface: Option<&str>) -> Result<String, ReadoutError> {
         shared::logical_address(interface)
     }
+
+    fn physical_address(&self, _: Option<&str>) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
