@@ -86,6 +86,10 @@ impl BatteryReadout for MacOSBatteryReadout {
             "Status property was not present in the dictionary that was returned from IOKit.",
         )))
     }
+
+    fn health(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 impl MacOSIOPMPowerSource {
@@ -204,6 +208,10 @@ impl GeneralReadout for MacOSGeneralReadout {
         }
     }
 
+    fn backlight(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn resolution(&self) -> Result<String, ReadoutError> {
         let displays = CGDisplay::active_displays();
         if let Err(e) = displays {
@@ -268,6 +276,10 @@ impl GeneralReadout for MacOSGeneralReadout {
         Ok(String::from("Aqua"))
     }
 
+    fn session(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn window_manager(&self) -> Result<String, ReadoutError> {
         Ok(String::from("Quartz Compositor"))
     }
@@ -423,6 +435,14 @@ impl MemoryReadout for MacOSMemoryReadout {
         Ok(((free_count * self.page_size as u64) / 1024) as u64)
     }
 
+    fn buffers(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn cached(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn reclaimable(&self) -> Result<u64, ReadoutError> {
         let vm_stats = MacOSMemoryReadout::mach_vm_stats()?;
         Ok((vm_stats.purgeable_count as u64 * self.page_size as u64 / 1024) as u64)
@@ -501,6 +521,10 @@ impl ProductReadout for MacOSProductReadout {
         Ok(String::from("Apple"))
     }
 
+    fn family(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn product(&self) -> Result<String, ReadoutError> {
         let mac_model = self
             .hw_model_ctl
@@ -525,10 +549,8 @@ impl PackageReadout for MacOSPackageReadout {
             }
         }
 
-        if extra::which("cargo") {
-            if let Some(c) = MacOSPackageReadout::count_cargo() {
-                packages.push((PackageManager::Cargo, c))
-            }
+        if let Some(c) = MacOSPackageReadout::count_cargo() {
+            packages.push((PackageManager::Cargo, c))
         }
 
         packages
@@ -586,9 +608,29 @@ impl NetworkReadout for MacOSNetworkReadout {
         MacOSNetworkReadout
     }
 
+    fn tx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn tx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn logical_address(&self, interface: Option<&str>) -> Result<String, ReadoutError> {
         shared::logical_address(interface)
     }
+
+    fn physical_address(&self, _: Option<&str>) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 fn macos_version_to_name(version: &NSOperatingSystemVersion) -> &'static str {
