@@ -73,16 +73,18 @@ impl BatteryReadout for NetBSDBatteryReadout {
 
             if envstat_out.is_empty() {
                 return Err(ReadoutError::MetricNotAvailable);
+            } else if envstat_out.contains("TRUE") {
+                return Ok(BatteryState::Charging);
             } else {
-                if envstat_out.contains("TRUE") {
-                    return Ok(BatteryState::Charging);
-                } else {
-                    return Ok(BatteryState::Discharging);
-                }
+                return Ok(BatteryState::Discharging);
             }
         }
 
-        Err(ReadoutError::Other(format!("envstat is not installed")))
+        Err(ReadoutError::Other("envstat is not installed".to_owned()))
+    }
+
+    fn health(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
     }
 }
 
@@ -100,7 +102,7 @@ impl KernelReadout for NetBSDKernelReadout {
         let osrelease = String::from_utf8(output.stdout)
             .expect("ERROR: \"sysctl\" process stdout was not valid UTF-8");
 
-        Ok(String::from(osrelease))
+        Ok(osrelease)
     }
 
     fn os_type(&self) -> Result<String, ReadoutError> {
@@ -112,7 +114,7 @@ impl KernelReadout for NetBSDKernelReadout {
         let osrelease = String::from_utf8(output.stdout)
             .expect("ERROR: \"sysctl\" process stdout was not valid UTF-8");
 
-        Ok(String::from(osrelease))
+        Ok(osrelease)
     }
 
     fn pretty_kernel(&self) -> Result<String, ReadoutError> {
@@ -169,13 +171,11 @@ impl GeneralReadout for NetBSDGeneralReadout {
             return Ok(vendor);
         }
 
-        let new_product: Vec<_> = new_product
+        Ok(new_product
             .split_whitespace()
             .into_iter()
             .unique()
-            .collect();
-
-        Ok(new_product.into_iter().join(" "))
+            .join(" "))
     }
 
     fn username(&self) -> Result<String, ReadoutError> {
@@ -188,7 +188,7 @@ impl GeneralReadout for NetBSDGeneralReadout {
         match hostname_cstr {
             Ok(hostname_cstr) => {
                 let hostname = hostname_cstr.to_str().unwrap_or("Unknown");
-                return Ok(String::from(hostname));
+                Ok(String::from(hostname))
             }
             Err(_e) => Err(ReadoutError::Other(String::from(
                 "Failed to retrieve hostname from 'gethostname'.",
@@ -258,7 +258,7 @@ impl GeneralReadout for NetBSDGeneralReadout {
                         .join("status");
 
                     if let Ok(status) = fs::read_to_string(path) {
-                        if let Some(name) = status.split_whitespace().nth(0) {
+                        if let Some(name) = status.split_whitespace().next() {
                             terminal_name = name.to_string();
                         }
                     }
@@ -273,7 +273,9 @@ impl GeneralReadout for NetBSDGeneralReadout {
         let terminal = terminal_name();
 
         if terminal.is_empty() {
-            return Err(ReadoutError::Other(format!("Could not to fetch terminal.")));
+            return Err(ReadoutError::Other(
+                "Could not to fetch terminal.".to_owned(),
+            ));
         }
 
         Ok(terminal)
@@ -353,6 +355,18 @@ impl MemoryReadout for NetBSDMemoryReadout {
         Ok(shared::get_meminfo_value("MemFree"))
     }
 
+    fn buffers(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn cached(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn reclaimable(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn used(&self) -> Result<u64, ReadoutError> {
         let total = self.total().unwrap();
         let free = self.free().unwrap();
@@ -375,7 +389,7 @@ impl ProductReadout for NetBSDProductReadout {
         let sysver = String::from_utf8(output.stdout)
             .expect("ERROR: \"sysctl\" process stdout was not valid UTF-8");
 
-        Ok(String::from(sysver))
+        Ok(sysver)
     }
 
     fn vendor(&self) -> Result<String, ReadoutError> {
@@ -387,7 +401,7 @@ impl ProductReadout for NetBSDProductReadout {
         let sysven = String::from_utf8(output.stdout)
             .expect("ERROR: \"sysctl\" process stdout was not valid UTF-8");
 
-        Ok(String::from(sysven))
+        Ok(sysven)
     }
 
     fn family(&self) -> Result<String, ReadoutError> {
@@ -399,7 +413,7 @@ impl ProductReadout for NetBSDProductReadout {
         let sysprod = String::from_utf8(output.stdout)
             .expect("ERROR: \"sysctl\" process stdout was not valid UTF-8");
 
-        Ok(String::from(sysprod))
+        Ok(sysprod)
     }
 }
 
@@ -410,21 +424,13 @@ impl PackageReadout for NetBSDPackageReadout {
 
     fn count_pkgs(&self) -> Vec<(PackageManager, usize)> {
         let mut packages = Vec::new();
-        // Instead of having a condition for each distribution.
-        // we will try and extract package count by checking
-        // if a certain package manager is installed
-        if extra::which("pkgin") {
-            match NetBSDPackageReadout::count_pkgin() {
-                Some(c) => packages.push((PackageManager::Pkgsrc, c)),
-                _ => (),
-            }
+
+        if let Some(c) = NetBSDPackageReadout::count_pkgin() {
+            packages.push((PackageManager::Pkgsrc, c));
         }
 
-        if extra::which("cargo") {
-            match NetBSDPackageReadout::count_cargo() {
-                Some(c) => packages.push((PackageManager::Cargo, c)),
-                _ => (),
-            }
+        if let Some(c) = NetBSDPackageReadout::count_cargo() {
+            packages.push((PackageManager::Cargo, c));
         }
 
         packages
@@ -458,7 +464,27 @@ impl NetworkReadout for NetBSDNetworkReadout {
         NetBSDNetworkReadout
     }
 
+    fn tx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn tx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn logical_address(&self, interface: Option<&str>) -> Result<String, ReadoutError> {
         shared::logical_address(interface)
     }
+
+    fn physical_address(&self, _: Option<&str>) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
