@@ -1,6 +1,5 @@
 mod sysinfo_ffi;
 
-use crate::extra;
 use crate::shared;
 use crate::traits::*;
 use byte_unit::AdjustedByte;
@@ -33,6 +32,18 @@ impl BatteryReadout for OpenWrtBatteryReadout {
     fn new() -> Self {
         OpenWrtBatteryReadout
     }
+
+    fn percentage(&self) -> Result<u8, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn status(&self) -> Result<BatteryState, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn health(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 impl KernelReadout for OpenWrtKernelReadout {
@@ -68,6 +79,14 @@ impl GeneralReadout for OpenWrtGeneralReadout {
         }
     }
 
+    fn backlight(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn resolution(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn machine(&self) -> Result<String, ReadoutError> {
         use std::io::{BufRead, BufReader};
         let file = fs::File::open("/proc/cpuinfo");
@@ -111,8 +130,24 @@ impl GeneralReadout for OpenWrtGeneralReadout {
         Ok(content.name)
     }
 
-    fn shell(&self, format: ShellFormat) -> Result<String, ReadoutError> {
-        shared::shell(format)
+    fn desktop_environment(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn session(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn window_manager(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn terminal(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn shell(&self, format: ShellFormat, kind: ShellKind) -> Result<String, ReadoutError> {
+        shared::shell(format, kind)
     }
 
     fn cpu_model_name(&self) -> Result<String, ReadoutError> {
@@ -172,6 +207,10 @@ impl GeneralReadout for OpenWrtGeneralReadout {
         }
     }
 
+    fn os_name(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn disk_space(&self) -> Result<(AdjustedByte, AdjustedByte), ReadoutError> {
         shared::disk_space(String::from("/"))
     }
@@ -253,11 +292,9 @@ impl PackageReadout for OpenWrtPackageReadout {
 
     fn count_pkgs(&self) -> Vec<(PackageManager, usize)> {
         let mut packages = Vec::new();
-        if extra::which("opkg") {
-            match OpenWrtPackageReadout::count_opkg() {
-                Some(c) => packages.push((PackageManager::Opkg, c)),
-                _ => (),
-            }
+
+        if let Some(c) = OpenWrtPackageReadout::count_opkg() {
+            packages.push((PackageManager::Opkg, c));
         }
 
         packages
@@ -270,16 +307,13 @@ impl OpenWrtPackageReadout {
     /// Including but not limited to:
     /// - [OpenWrt](https://openwrt.org)
     fn count_opkg() -> Option<usize> {
-        use std::io::{BufRead, BufReader};
         let mut count: usize = 0;
         let file = fs::File::open("/usr/lib/opkg/status");
         if let Ok(content) = file {
             let reader = BufReader::new(content);
-            for line in reader.lines() {
-                if let Ok(l) = line {
-                    if l.starts_with("Package:") {
-                        count += 1
-                    }
+            for line in reader.lines().flatten() {
+                if line.starts_with("Package:") {
+                    count += 1
                 }
             }
 
@@ -295,7 +329,27 @@ impl NetworkReadout for OpenWrtNetworkReadout {
         OpenWrtNetworkReadout
     }
 
+    fn tx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn tx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn logical_address(&self, interface: Option<&str>) -> Result<String, ReadoutError> {
         shared::logical_address(interface)
     }
+
+    fn physical_address(&self, _: Option<&str>) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
