@@ -1,7 +1,6 @@
 #![allow(dead_code)]
 #![allow(unused_imports)]
 
-use crate::extra;
 use crate::traits::{ReadoutError, ShellFormat, ShellKind};
 
 use std::fs::read_dir;
@@ -61,7 +60,7 @@ pub(crate) fn desktop_environment() -> Result<String, ReadoutError> {
                 )));
             }
 
-            Ok(extra::ucfirst(de))
+            Ok(crate::extra::ucfirst(de))
         }
         Err(_) => Err(ReadoutError::Other(String::from(
             "You appear to be only running a window manager.",
@@ -77,14 +76,14 @@ pub(crate) fn desktop_environment() -> Result<String, ReadoutError> {
 )))]
 pub(crate) fn session() -> Result<String, ReadoutError> {
     match env::var("XDG_SESSION_TYPE") {
-        Ok(s) => Ok(extra::ucfirst(s)),
+        Ok(s) => Ok(crate::extra::ucfirst(s)),
         Err(_) => Err(ReadoutError::Other(String::from(
             "No graphical session detected.",
         ))),
     }
 }
 
-#[cfg(target_os = "linux")]
+#[cfg(all(target_os = "linux", not(feature = "openwrt")))]
 pub(crate) fn window_manager() -> Result<String, ReadoutError> {
     use crate::winman::*;
 
@@ -120,9 +119,9 @@ pub(crate) fn resolution() -> Result<String, ReadoutError> {
     let mut resolution: Vec<String> = vec![];
     if let Ok(conn) = x11rb::connect(None) {
         let screens = &conn.0.setup().roots;
-        for scr in screens {
-            let width = scr.width_in_pixels;
-            let height = scr.height_in_pixels;
+        for s in screens {
+            let width = s.width_in_pixels;
+            let height = s.height_in_pixels;
             resolution.push(width.to_string() + "x" + &height.to_string())
         }
 
@@ -194,17 +193,23 @@ pub(crate) fn shell(shorthand: ShellFormat, kind: ShellKind) -> Result<String, R
             )))
         }
         ShellKind::Current => {
-            let path = PathBuf::from("/proc")
-                .join(unsafe { libc::getppid() }.to_string())
-                .join("comm");
+            if cfg!(target_os = "macos") {
+                Err(ReadoutError::Other(String::from(
+                    "Retrieving the current shell is not supported on macOS.",
+                )))
+            } else {
+                let path = PathBuf::from("/proc")
+                    .join(unsafe { libc::getppid() }.to_string())
+                    .join("comm");
 
-            if let Ok(shell) = read_to_string(path) {
-                return Ok(shell);
+                if let Ok(shell) = read_to_string(path) {
+                    return Ok(shell);
+                }
+
+                Err(ReadoutError::Other(String::from(
+                    "Unable to read current shell.",
+                )))
             }
-
-            Err(ReadoutError::Other(String::from(
-                "Unable to read current shell.",
-            )))
         }
     }
 }
@@ -220,7 +225,7 @@ pub(crate) fn cpu_model_name() -> String {
                 if line.starts_with("model name") {
                     return line
                         .replace("model name", "")
-                        .replace(":", "")
+                        .replace(':', "")
                         .trim()
                         .to_string();
                 }
@@ -324,24 +329,12 @@ pub(crate) fn logical_address(interface: Option<&str>) -> Result<String, Readout
 }
 
 pub(crate) fn count_cargo() -> Option<usize> {
-    if let Ok(cargo_home) = std::env::var("CARGO_HOME") {
-        let bin = PathBuf::from(cargo_home).join("bin");
-        if bin.exists() {
-            if let Ok(read_dir) = read_dir(bin) {
-                return Some(read_dir.count());
-            }
+    if let Ok(cargo_home) = home::cargo_home() {
+        let bin = cargo_home.join("bin");
+        if let Ok(read_dir) = read_dir(bin) {
+            return Some(read_dir.count());
         }
     }
 
-    if let Ok(home) = std::env::var("HOME") {
-        let bin = PathBuf::from(home).join(".cargo").join("bin");
-        if bin.exists() {
-            if let Ok(read_dir) = read_dir(bin) {
-                return Some(read_dir.count());
-            }
-        }
-
-        return None;
-    }
     None
 }
