@@ -1,4 +1,3 @@
-use crate::extra;
 use crate::traits::*;
 use std::collections::HashMap;
 use winreg::enums::*;
@@ -53,6 +52,10 @@ impl BatteryReadout for WindowsBatteryReadout {
             ))),
         };
     }
+
+    fn health(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 impl WindowsBatteryReadout {
@@ -107,6 +110,22 @@ impl MemoryReadout for WindowsMemoryReadout {
         Ok(memory_status.ullTotalPhys / 1024u64)
     }
 
+    fn free(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn buffers(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn cached(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn reclaimable(&self) -> Result<u64, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn used(&self) -> Result<u64, ReadoutError> {
         let memory_status = WindowsMemoryReadout::get_memory_status()?;
         Ok((memory_status.ullTotalPhys - memory_status.ullAvailPhys) / 1024u64)
@@ -137,6 +156,14 @@ impl GeneralReadout for WindowsGeneralReadout {
         WindowsGeneralReadout
     }
 
+    fn backlight(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn resolution(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn username(&self) -> Result<String, ReadoutError> {
         let mut size = 0;
         unsafe { GetUserNameA(PSTR(std::ptr::null_mut()), &mut size) };
@@ -222,6 +249,30 @@ impl GeneralReadout for WindowsGeneralReadout {
         Ok(str)
     }
 
+    fn distribution(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn desktop_environment(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn session(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn window_manager(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn terminal(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn shell(&self, _shorthand: ShellFormat, _: ShellKind) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn cpu_model_name(&self) -> Result<String, ReadoutError> {
         let hklm = RegKey::predef(HKEY_LOCAL_MACHINE);
         let central_processor =
@@ -232,6 +283,18 @@ impl GeneralReadout for WindowsGeneralReadout {
         Ok(processor_name)
     }
 
+    fn cpu_usage(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn cpu_physical_cores(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn cpu_cores(&self) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn uptime(&self) -> Result<usize, ReadoutError> {
         let tick_count = unsafe { GetTickCount64() };
         let duration = std::time::Duration::from_millis(tick_count);
@@ -268,6 +331,12 @@ impl GeneralReadout for WindowsGeneralReadout {
                 .to_string(),
         ))
     }
+
+    fn disk_space(
+        &self,
+    ) -> Result<(byte_unit::AdjustedByte, byte_unit::AdjustedByte), ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
 
 pub struct WindowsProductReadout {
@@ -299,6 +368,10 @@ impl ProductReadout for WindowsProductReadout {
         }
     }
 
+    fn family(&self) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn product(&self) -> Result<String, ReadoutError> {
         match &self.model {
             Some(v) => Ok(v.clone()),
@@ -322,12 +395,12 @@ impl PackageReadout for WindowsPackageReadout {
     /// - cargo
     fn count_pkgs(&self) -> Vec<(PackageManager, usize)> {
         let mut packages = Vec::new();
-        if extra::which("cargo") {
-            if let Some(c) = WindowsPackageReadout::count_cargo() {
-                packages.push((PackageManager::Cargo, c))
-            }
+        if let Some(c) = WindowsPackageReadout::count_cargo() {
+            packages.push((PackageManager::Cargo, c));
+        }
+        if let Some(c) = WindowsPackageReadout::count_scoop() {
+            packages.push((PackageManager::Scoop, c));
         }
-
         packages
     }
 }
@@ -336,6 +409,16 @@ impl WindowsPackageReadout {
     fn count_cargo() -> Option<usize> {
         crate::shared::count_cargo()
     }
+
+    fn count_scoop() -> Option<usize> {
+        if let Ok(path) = which::which("scoop") {
+            if let Ok(dir) = path.join("../../apps").read_dir() {
+                return Some(dir.count() - 1); // One entry belongs to scoop itself
+            }
+        }
+
+        None
+    }
 }
 
 pub struct WindowsNetworkReadout;
@@ -345,6 +428,22 @@ impl NetworkReadout for WindowsNetworkReadout {
         WindowsNetworkReadout
     }
 
+    fn tx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn tx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_bytes(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
+    fn rx_packets(&self, _: Option<&str>) -> Result<usize, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
+
     fn logical_address(&self, interface: Option<&str>) -> Result<String, ReadoutError> {
         match interface {
             Some(it) => {
@@ -365,4 +464,8 @@ impl NetworkReadout for WindowsNetworkReadout {
             "Unable to get local IP address.".to_string(),
         ))
     }
+
+    fn physical_address(&self, _: Option<&str>) -> Result<String, ReadoutError> {
+        Err(ReadoutError::NotImplemented)
+    }
 }
